##############################################################################
#
# Copyright (c) 2002 Nik Kim <fafhrd@mindless.com> All Rights Reserved.
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl

try:
    _lo_attrs = 1
    from Products.CMFCore.LocaleAttributes import LocaleAttributes
except:
    _lo_attrs = 0
    class LocaleAttributes: pass


class CMFField(PortalContent, DefaultDublinCoreImpl):
    """ cmf formulator Field """

    meta_type = 'CMF Form Field'

    __implements__ = ( PortalContent.__implements__,
                       DefaultDublinCoreImpl.__implements__ )

    if _lo_attrs:
        __locale_attrs__ = {'title':'', 'description':'', 'default': ''}
    else:
        default = ''
        
    security = ClassSecurityInfo()

    manage_options = PortalContent.manage_options
    
    hidden = 0
    value = ''
    css_class = ''
    alternate_name = ''
    width = 25
    required = 0
    extra = ''
    external_validator = ''
    message_values = {}

    def __init__(self, id, title='', **kwargs):
        self.id = id
        self.title = title
        DefaultDublinCoreImpl.__init__(self, title)

        self.edit(**kwargs)

    def manage_beforeDelete(self, item, container):
        """ """
        try:
            container.field_removed(item)
        except Exception, msg:
            print str(msg), container.Type()

    security.declarePrivate('manage_afterAdd')
    def manage_afterAdd(self, item, container):
        """What happens when we add a field.
        """
        # update group info in form
        container.field_added(item)

    security.declareProtected(ModifyPortalContent, 'edit')
    def edit(self, default='', hidden=0, required=0, css_class='',
             alternate_name = '', extra='', **kwargs):
        """ """
        self.default = default
        self.hidden = hidden
        self.required = required
        self.css_class = css_class
        self.alternate_name = alternate_name
        self.extra = extra
        for key, val in kwargs.items():
            setattr(self, key, val)

    security.declareProtected(View, 'Hidden')
    def Hidden(self):
        return self.hidden
    
    security.declareProtected(View, 'Required')
    def Required(self):
        return self.required

    security.declareProtected('Access contents information', 'get_value')
    def get_value(self, id):
        """ """
        return getattr(self, id, None)

    security.declarePrivate('_get_default')
    def _get_default(self, value, REQUEST):
        if value is not None:
            return value
        try:
            return self.REQUEST.form[self.id]
        except:
            return self.default

    security.declarePrivate('_render_helper')
    def _render_helper(self, value, REQUEST):
        value = self._get_default(value, REQUEST)
        if self.hidden:
            return self.widget.render_hidden(self, self.id, value, REQUEST)
        else:
            return self.widget.render(self, self.id, value, REQUEST)

    security.declareProtected('View', 'render')
    def render(self, value=None, REQUEST=None):
        """ """
        return self._render_helper(value, REQUEST)

    security.declarePrivate('_validate_helper')
    def _validate_helper(self, REQUEST):
        value = self.validator.validate(self, self.id, REQUEST)
        # now call external validator after all other validation
        external_validator = self.external_validator
        if external_validator and not external_validator(value, REQUEST):
            self.validator.raise_error('external_validator_failed', self)
        return value
    
    security.declareProtected(View, 'validate')
    def validate(self, REQUEST):
        """Validate/transform the field."""
        return self._validate_helper(REQUEST)
    
    security.declareProtected(View, 'validate_sub_field')
    def validate_sub_field(self, id, REQUEST):
        """Validates a subfield (as part of field validation).
        """
        return self.sub_form.get_field(id)._validate_helper(
            "subfield_%s_%s" % (self.id, id), REQUEST)

InitializeClass(CMFField)
