##############################################################################
#
# Copyright (c) 2002 Nik Kim <fafhrd@mindless.com> All Rights Reserved.
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

import binascii, locale, traceback
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl

factory_type_information = (
    {   'id'             : 'MailStorage',
        'meta_type'      : 'DataStorage',
        'description'    : """Mail Storage""",
        'icon'           : 'Form.gif',
        'product'        : 'CMFFormulator',
        'factory'        : 'manage_addMailStorage',
        'immediate_view' : 'mstorage_edit_form',
        'filter_content_types' : 0,
        'actions'        :
      ( { 'id'         : 'view',
          'name'       : 'View',
          'action'     : 'mstorage_view',
          'permissions': (View,),
          'category'   : 'object',
          },
        {'id'         : 'edit',
         'name'       : 'Edit',
         'action'     : 'mstorage_edit_form',
         'permissions': (ModifyPortalContent,),
         'category'   : 'object',
         },
        {'id'         : 'metadata',
         'name'       : 'Metadata',
         'action'     : 'metadata_edit_form',
         'permissions': (ModifyPortalContent, ),
         'category'   : 'object',
         },

        )
        },
    )



class MailStorage(PortalContent, DefaultDublinCoreImpl):
    """ """
    meta_type = 'DataStorage'

    __implements__ = ( PortalContent.__implements__,
                       DefaultDublinCoreImpl.__implements__ )

    security = ClassSecurityInfo()

    email_to = ''
    email_from = ''
    encoding = 'base64'
    count = 0

    _dataStorage = 1

    def __init__(self, id, title=''):
        self.id = id
        self.title = title
    
    security.declareProtected(ModifyPortalContent, 'edit')
    def edit(self, email_to, email_from):
        """ edit form """
        self.email_to = email_to
        self.email_from = email_from
        
    security.declarePrivate('append')
    def append(self, source, record={}, dictionary={}, **kwargs):
        """ """
        try:
            enc = locale.getlocale()[1]
            if enc == 'utf': enc='utf-8'
            subj = '=?%s?B?%s?=' % (enc,
                binascii.b2a_base64(source.Title() or '[No subject]').strip())
            if not self.email_to: return
            mh = self.portal_url.getPortalObject().MailHost
            msg = self.mstorage_message(Title=source.Title(), data=record,
                                        dictionary=dictionary)
            mh.send(msg, self.email_to, self.email_from, subj, self.encoding)
        except:
            traceback.print_exc()

        self.count += 1

    security.declareProtected(View, 'EMailTo')
    def EMailTo(self):
        """ edit form """
        return self.email_to

    security.declareProtected(View, 'EMailFrom')
    def EMailFrom(self):
        """ edit form """
        return self.email_from

    security.declareProtected(View, 'RecordCount')
    def RecordCount(self):
        """ """
        return self.count

InitializeClass(MailStorage)


def manage_addMailStorage(self, id, title="", REQUEST=None):
    """Add form to folder. """
    id = self._setObject(id, MailStorage(id, title))
    
    if REQUEST is not None:
        return self.folder_contents( # XXX: ick!
            self, REQUEST, portal_status_message="MailStorage form added")

